#!/usr/bin/perl
#
# Title: 	Delta Calculator
#
# File: 	deltacalc.pl
#
# Version: 	1.0
#
# Written by:	raffael.marty@arcsight.com (ram)
#
# Comments:	Go throught the DB and update it with delta information, i.e. it
# 		adds delta time to all the connections. This will show automatic
# 		scans
#
# Usage:	./deltacalc.pl
#
# URL:		http://afterglow.sourceforge.net
#
# Changes:	
# 
# 10/17/04	Initial Version by ram
#
###############################################################################/

use strict;
use DBI; 
use Date::Manip qw(ParseDate UnixDate);
use Date::Calc qw(Delta_DHMS);

my $dbh = DBI->connect('DBI:mysql:tcpdump:localhost', 'root', 'pass')
	or die "Couldn't connect to database: " . DBI->errstr;

# prepare some queries
my $query = qq{select id, timestamp, sourceip, sourceport, destip, destport from sans order by sourceip,sourceport,destip,destport,timestamp};
my $update = qq{update sans set delta=? where id=?};
my $sth = $dbh->prepare($query) or die ("SQL error: ").$dbh->errstr;
my $sth1 = $dbh->prepare($update) or die ("SQL error: ").$dbh->errstr;

my (@old_date, $old_sip, $old_dip, $old_sport, $old_dport);

my $res = $sth->execute() or die ("SQL error: ").$dbh->errstr;

while (my @data = $sth->fetchrow_array()) {

	my $values = ParseDate($data[1]);
	my @date = UnixDate($values, "%Y", "%m", "%d", "%H", "%M", "%S");

	if ( ($data[2] == $old_sip) && ($data[3] == $old_sport)
		&& ($data[4] == $old_dip) && ($data[5] == $old_dport) ) {
		
		# same connection, update delta

		my @diff = Delta_DHMS($old_date[0], $old_date[1], $old_date[2], 
			$old_date[3], $old_date[4], $old_date[5], 
			$date[0], $date[1], $date[2], 
			$date[3], $date[4], $date[5]);
	
		my $diff = $diff[1]*3600+$diff[2]*60+$diff[3];

		#print "$data[0]; delta: $diff\n";

		$sth1->execute($diff, $data[0]);
		

	} 

	# save old values
	@old_date = @date;
	$old_sip = $data[2];
	$old_sport = $data[3];
	$old_dip = $data[4];
	$old_dport = $data[5];
}

$dbh->disconnect();

exit;

sub mac() {
	my ($input) = @_; 
	my $output;
	my @splitter = split(":",$input);
	foreach my $sm (@splitter) {
		if (length($sm)==1) {
			$sm="0".$sm;
		}
		$output .= $sm.":";
	}
	$output =~ s/:$//;
	return lc($output);
}		
